@@ -27,8 +27,6 @@
 
 #include <garcon-gtk/garcon-gtk-menu.h>
 
-#define STR_IS_EMPTY(str) ((str) == NULL || *(str) == '\0')
-
 
 /**
  * SECTION: garcon-gtk-menu
@@ -333,108 +331,46 @@ garcon_gtk_menu_show (GtkWidget *widget)
 
 
 
-static void
-garcon_gtk_menu_append_quoted (GString     *string,
-                               const gchar *unquoted)
-{
-  gchar *quoted;
-
-  quoted = g_shell_quote (unquoted);
-  g_string_append (string, quoted);
-  g_free (quoted);
-}
-
-
-
 static void
 garcon_gtk_menu_item_activate_real (GtkWidget            *mi,
                                     GarconMenuItem       *item,
                                     GarconMenuItemAction *action)
 {
-  GString      *string;
-  const gchar  *command;
-  const gchar  *p;
-  const gchar  *tmp;
+  gchar        *command, *uri;
   gchar       **argv;
+  const gchar  *icon;
   gboolean      result = FALSE;
-  gchar        *uri;
   GError       *error = NULL;
 
   g_return_if_fail (GTK_IS_WIDGET (mi));
   g_return_if_fail (GARCON_IS_MENU_ITEM (item));
 
   if (action != NULL)
-    {
-      command = garcon_menu_item_action_get_command (action);
-    }
+    command = (gchar*) garcon_menu_item_action_get_command (action);
   else
-    {
-      command = garcon_menu_item_get_command (item);
-    }
+    command = (gchar*) garcon_menu_item_get_command (item);
 
-  if (STR_IS_EMPTY (command))
+  if (xfce_str_is_empty (command))
     return;
 
-  string = g_string_sized_new (100);
-
-  if (garcon_menu_item_requires_terminal (item))
-    g_string_append (string, "exo-open --launch TerminalEmulator ");
-
   /* expand the field codes */
-  for (p = command; *p != '\0'; ++p)
-    {
-      if (G_UNLIKELY (p[0] == '%' && p[1] != '\0'))
-        {
-          switch (*++p)
-            {
-            case 'f': case 'F':
-            case 'u': case 'U':
-              /* TODO for dnd, not a regression, xfdesktop never had this */
-              break;
-
-            case 'i':
-              tmp = garcon_menu_item_get_icon_name (item);
-              if (!STR_IS_EMPTY (tmp))
-                {
-                  g_string_append (string, "--icon ");
-                  garcon_gtk_menu_append_quoted (string, tmp);
-                }
-              break;
-
-            case 'c':
-              tmp = garcon_menu_item_get_name (item);
-              if (!STR_IS_EMPTY (tmp))
-                garcon_gtk_menu_append_quoted (string, tmp);
-              break;
-
-            case 'k':
-              uri = garcon_menu_item_get_uri (item);
-              if (!STR_IS_EMPTY (uri))
-                garcon_gtk_menu_append_quoted (string, uri);
-              g_free (uri);
-              break;
-
-            case '%':
-              g_string_append_c (string, '%');
-              break;
-            }
-        }
-      else
-        {
-          g_string_append_c (string, *p);
-        }
-    }
+  icon = garcon_menu_item_get_icon_name (item);
+  uri = garcon_menu_item_get_uri (item);
+  command = xfce_expand_desktop_entry_field_codes (command, NULL, icon,
+                                                   garcon_menu_item_get_name (item),
+                                                   uri,
+                                                   garcon_menu_item_requires_terminal (item));
+  g_free (uri);
 
   /* parse and spawn command */
-  if (g_shell_parse_argv (string->str, NULL, &argv, &error))
+  if (g_shell_parse_argv (command, NULL, &argv, &error))
     {
       result = xfce_spawn_on_screen (gtk_widget_get_screen (mi),
                                      garcon_menu_item_get_path (item),
                                      argv, NULL, G_SPAWN_SEARCH_PATH,
                                      garcon_menu_item_supports_startup_notification (item),
                                      gtk_get_current_event_time (),
-                                     garcon_menu_item_get_icon_name (item),
-                                     &error);
+                                     icon, &error);
 
       g_strfreev (argv);
     }
@@ -445,7 +381,7 @@ garcon_gtk_menu_item_activate_real (GtkWidget            *mi,
       g_error_free (error);
     }
 
-  g_string_free (string, TRUE);
+  g_free (command);
 }
 
 
@@ -548,7 +484,7 @@ garcon_gtk_menu_item_drag_begin (GarconMenuItem *item,
   g_return_if_fail (GARCON_IS_MENU_ITEM (item));
 
   icon_name = garcon_menu_item_get_icon_name (item);
-  if (!STR_IS_EMPTY (icon_name))
+  if (!xfce_str_is_empty (icon_name))
     gtk_drag_set_icon_name (drag_context, icon_name, 0, 0);
 }
 
@@ -715,22 +651,7 @@ garcon_gtk_menu_create_menu_item (gboolean     show_menu_icons,
                                   const gchar *icon_name)
 {
   GtkWidget *mi;
-  GtkWidget *box;
   GtkWidget *image;
-  GtkWidget *label;
-
-  /* create item */
-  mi = gtk_menu_item_new ();
-  label = gtk_label_new (name);
-  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-#if GTK_CHECK_VERSION (3, 0, 0)
-  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
-  gtk_widget_set_halign (label, GTK_ALIGN_START);
-#else
-  box = gtk_hbox_new (FALSE, 0);
-  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5f);
-#endif
 
   if (show_menu_icons)
     {
@@ -742,11 +663,7 @@ garcon_gtk_menu_create_menu_item (gboolean     show_menu_icons,
       image = gtk_image_new ();
     }
 
-  /* Add the image and label to the box, add the box to the menu item */
-  gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 6);
-  gtk_widget_show_all (box);
-  gtk_container_add (GTK_CONTAINER (mi), box);
+  mi = xfce_gtk_image_menu_item_new (name, NULL, NULL, NULL, NULL, image, NULL);
 
   return mi;
 }
@@ -876,7 +793,7 @@ garcon_gtk_menu_add (GarconGtkMenu *menu,
             continue;
 
           icon_name = garcon_menu_item_get_icon_name (li->data);
-          if (STR_IS_EMPTY (icon_name))
+          if (xfce_str_is_empty (icon_name))
             icon_name = "applications-other";
 
           /* build the menu item */
@@ -910,7 +827,7 @@ garcon_gtk_menu_add (GarconGtkMenu *menu,
           if (menu->priv->show_tooltips)
             {
               comment = garcon_menu_item_get_comment (li->data);
-              if (!STR_IS_EMPTY (comment))
+              if (!xfce_str_is_empty (comment))
                 gtk_widget_set_tooltip_text (mi, comment);
             }
 
@@ -926,7 +843,7 @@ garcon_gtk_menu_add (GarconGtkMenu *menu,
 
           /* doesn't happen, but anyway... */
           command = garcon_menu_item_get_command (li->data);
-          if (STR_IS_EMPTY (command))
+          if (xfce_str_is_empty (command))
             gtk_widget_set_sensitive (mi, FALSE);
 
           /* atleast 1 visible child */
@@ -957,7 +874,7 @@ garcon_gtk_menu_add (GarconGtkMenu *menu,
               name = garcon_menu_element_get_name (li->data);
 
               icon_name = garcon_menu_element_get_icon_name (li->data);
-              if (STR_IS_EMPTY (icon_name))
+              if (xfce_str_is_empty (icon_name))
                 icon_name = "applications-other";
 
               /* build the menu item */
@@ -1086,7 +1003,7 @@ garcon_gtk_menu_set_menu (GarconGtkMenu *menu,
  * The caller is responsible to releasing the returned #GarconMenu
  * using g_object_unref().
  *
- * Returns: the #GarconMenu for @menu.
+ * Returns: (transfer full): the #GarconMenu for @menu.
  **/
 GarconMenu *
 garcon_gtk_menu_get_menu (GarconGtkMenu *menu)
@@ -1256,9 +1173,10 @@ garcon_gtk_menu_get_show_desktop_actions (GarconGtkMenu *menu)
  * garcon_gtk_menu_get_desktop_actions_menu:
  * @item  : A #GarconMenuItem
  *
- * Return value: a #GtkMenu holding all actions described in the desktop file
- * as menu items. Note that application icons are never shown on the
- * action menu items.
+ * Application icons are never shown on the action menu items.
+ *
+ * Returns: (transfer full): a #GtkMenu holding all actions described
+ * in the desktop file as menu items.
  **/
 GtkMenu *
 garcon_gtk_menu_get_desktop_actions_menu (GarconMenuItem *item)
